/* LayoutPopup.java
 *
 * Jim McBeath
 */

package jimmc.roots;

import jimmc.treepar.LayoutHitText;

import java.awt.MenuItem;
import java.awt.PopupMenu;

/** A popup menu for use with a LayoutPanel.
 * It includes a select command that picks the key value (id field) out
 * of the text under the cursor.
 */
public class LayoutPopup extends PopupMenu {
	TextPopupListener w;
	MenuPush selectMenu;
	String id;

	/** Create the popup. */
	public LayoutPopup(TextPopupListener ww) {
		super("------");
		w = ww;
		addMenuItems();
	}

	/** Add the menu items to the popup. */
	protected void addMenuItems() {
		add(selectMenu=new MenuPush("Select") {
			public void action() {
				w.processSelect(id);
			}
		});
	}

	/** Enable or disable the menus that depend on a valid ID. */
	protected void enableIdMenus(boolean enable) {
		if (selectMenu!=null)
			selectMenu.setEnabled(enable);
	}

	/** Set stuff in the popup menu based on x,y position */
	protected void setRecord(LayoutPanelWithPopup tf, int x,int y) {
		id = getID(tf,x,y);
		if (id==null) {
			enableIdMenus(false);
			id = "????";
		} else {
			enableIdMenus(true);
		}
		setLabel(id);
	}

	/** Get the ID (key) value from the text under the cursor. */
	private String getID(LayoutPanelWithPopup tf, int x, int y) {
		LayoutHitText h = tf.getHitText(x,y);
			/* Get the text line around the specified point
			 * and the position of the point within that string. */
		if (h==null)
			return null;	/* no text there */
		return getWordAround(h.text,h.position);
			/* Extract the key string from it. */
	}

	/** Get the word in the given single line 
	 * around the specified position. */
	protected String getWordAround(String text,int pos) {
		/* Constrain the text to be between spaces */
		int bPos = text.lastIndexOf(' ',pos);
		if (bPos<0)
			bPos = 0;
		int ePos = text.indexOf(' ',pos);
		if (ePos<0)
			ePos = text.length();

		/* Look for enclosing square brackets. */
		int pos0 = text.lastIndexOf('[',pos);
		int pos1 = text.indexOf(']',pos);
		if (pos0>=bPos && pos1>=0 && pos1<=ePos) {
			/* found enclosing square brackets,
			 * between is the ID string. */
			return text.substring(pos0+1,pos1);
		}

		/* Look for enclosing '@' chars (as in gedcom). */
		pos0 = text.lastIndexOf('@',pos);
		pos1 = text.indexOf('@',pos);
		if (pos0>=bPos && pos1>=0 && pos1<=ePos) {
			/* found enclosing "@" chars,
			 * between is the ID string. */
			return text.substring(pos0+1,pos1);
		}

		/* Didn't find matching square brackets, look at start of line*/
		int sPos = text.indexOf(' ');
		int s2Pos = text.indexOf('\t');
		if (s2Pos>=0 && s2Pos<sPos)
			sPos = s2Pos;
		if (sPos>pos) {
			return text.substring(0,sPos);
				/* Well, give that string a try */
		}

		return null;	/* can't figure it out, give up */
	}
}

/* end */
