/* ListModel.java
 *
 * Jim McBeath, June 7, 1998
 */

package jimmc.roots;

/** A model for a single record from the database.
 */
public class ListModel {
	/** Our database model. */
	protected DatabaseModel db;
	/** Our key list. */
	protected String[] keys;
	/** Our base database. */
	protected RootsBase base;

	/** Create a new model. */
	public ListModel(DatabaseModel db, String[] keys) {
		this.db = db;
		this.keys = keys;
		base = db.getBase();
	}

	/** Get the count of keys. */
	public int getCount() {
		return keys.length;
	}

	/** Get the command we use to generate our display. */
	public String getDisplayCommand(String format, boolean includeKeys) {
		StringBuffer sb = new StringBuffer();
		sb.append("print ");
		if (includeKeys)
			sb.append("-k ");
		if (format.equals("list"))
			sb.append("-f list -r oneLine");
		else if (format.equals("gedcom"))
			sb.append("-f list -r gedcom");
		else if (format.equals("ncv"))
			sb.append("-r ncv");
		for (int i=0; i<keys.length; i++)
			sb.append(' ').append(keys[i]);   /* our records */
		return sb.toString();
	}

	/** Get the long text string based on format. */
	public String getOutputText(String format, boolean includeKeys) {
		String cmd = getDisplayCommand(format,includeKeys);
		return db.doCommand(cmd);
	}
}

/* end */
