/* Copyright 1997 Jim McBeath
 *
 * Jim McBeath, January 13, 1997
 */

package jimmc.roots;

import java.io.*;
import java.util.StringTokenizer;
import java.util.Vector;

/** Ncv stands for Name-Colon-Value, referring to the syntax of the
 * data file it is used to parse.
 * A single instance of Ncv represents one name-value pair.
 * Static methods provide support for parsing.
 */
public class Ncv {
	public String name;
	public String value;	/* may contain newlines if multiple lines */
	public int numLines; /* number of lines of text (newlines+1) in value */

	/** Create an Ncv from an array of Strings and a starting point. */
	public Ncv(String lines[], int startLine) {
		 name = getName(lines[startLine]);
		 value = lines[startLine].substring(name.length()+1);
		 int n;
		 for (n = startLine+1; n<lines.length; n++) {
			if (lines[n].startsWith("+"))
				value = value + "\n" + lines[n].substring(1);
			else
				break;	/* not a continuation line */
		 }
		 numLines = n - startLine;
	}

	/* Read a set of input lines for one record from the database */
	public static String[] readLines(LineNumberReader li)
					throws IOException {
		String line;
		Vector linev = new Vector();
		while ((line=li.readLine())!=null) {
			if (line.length()==0)
				break;
			if (line.startsWith(":"))	//line starting with :
				continue;		// are comment lines
			linev.addElement(line);
		}
		String lines[] = new String[linev.size()];
		linev.copyInto(lines);
		return lines;
	}

	/** Look for a particular name and return its value */
	protected static String getRecordValue(String lines[],String name) {
		for (int i=0; i<lines.length; i++) {
			if (lines[i].startsWith(name+":",0))
				return lines[i].substring(name.length()+1);
		}
		return null;	/* didn't find it */
	}
	/** Return the first word on the key line of the record */
	protected static String getKey(String line) {
		StringTokenizer st = new StringTokenizer(line);
		if (st.countTokens()==0)
			return null;
		return st.nextToken();
	}
	/** Return the first word on a colon line of the record */
	protected static String getName(String line) {
		StringTokenizer st = new StringTokenizer(line,":");
		return st.nextToken();
	}
}

/* end */
