/* RootsFrame.java
 *
 * Jim McBeath, June 11, 1998
 */

package jimmc.roots;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JOptionPane;
import javax.swing.JTextArea;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.FileWriter;
import java.io.IOException;

/** A general Frame to contain the methods common to our views.
 */
public class RootsFrame extends JFrame {
	/** Create a frame without a name. */
	public RootsFrame() {
		super();
	}

	/** Create a frame with a name. */
	public RootsFrame(String name) {
		super(name);
	}

	/** Add a window listener to close our window. */
	protected void addWindowListener() {
		WindowAdapter a = new WindowAdapter() {
			public void windowClosing(WindowEvent ev) {
				processClose();
			}
		};
		addWindowListener(a);
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
	}

	/** Create the Help menu. */
	protected JMenu createHelpMenu() {
		JMenu m = new JMenu("Help");
		MenuAction mi;
		mi = new MenuAction("About Roots...") {
			public void action() {
				processHelpAbout();
			}
		};
		m.add(mi);
		return m;
	}

	/** Process the Help->About menu command. */
	protected void processHelpAbout() {
		AboutWindow.showAboutWindow(this);
	}

	/** Close this window. */
	protected void processClose() {
		setVisible(false);
		dispose();
	}

	/** Put up an error dialog. */
	public void errorDialog(String errmsg) {
		JOptionPane.showMessageDialog(this,errmsg,"Error",
			JOptionPane.ERROR_MESSAGE);
	}

	/** Put up an info dialog. */
	public void infoDialog(String infomsg) {
		if (infomsg.length()>80) {
			//It's a long string, so rather than putting it up as
			//a single string, we put it up as a text box with
			//wraparound turned on.
			JTextArea tx = new JTextArea(infomsg);
			tx.setEditable(false);
			tx.setLineWrap(true);
			JOptionPane.showMessageDialog(this,tx,"Info",
				JOptionPane.INFORMATION_MESSAGE);
			return;
		}
		JOptionPane.showMessageDialog(this,infomsg,"Info",
			JOptionPane.INFORMATION_MESSAGE);
	}

	/** Put up a string dialog. */
	public String stringDialog(String prompt) {
		return JOptionPane.showInputDialog(prompt);
	}

	/** Put up a file dialog. */
	public String fileDialog(String prompt) {
		/* TBD - use a file browser. */
		return JOptionPane.showInputDialog(prompt);
	}

	/* TBD - add saveTextToFile(String text) that asks here for
	 * the output file.
	 */

	/** Save the specified text string to the specified file. */
	public void saveTextToFile(String text, String filename) {
		/* TBD - if file already exists, complain */
		try {
			FileWriter w = new FileWriter(filename);
			w.write(text);
			w.close();
			/* TBD - notify somehow that it was written OK,
			 * perhaps log to a database view?
			 * Or return a status code so caller can do so.
			 */
		} catch (IOException e) {
			errorDialog("Error writing file '"+filename+"'");
		}
	}
}

/* end */
