/* Copyright 1997 Jim McBeath
 *
 * Jim McBeath, January 12, 1997
 */

package jimmc.roots;

import jimmc.treepar.Layout;

import java.io.*;

/** RootsUnion represents a marriage or other pairing.
 */

public class RootsUnion extends RootsRecord {
	String familyName;
	String husbandString;
	String wifeString;
	RootsPerson husband;	/* father of the children, married or not */
	RootsPerson wife;	/* mother of the children, married or not */
	RootsEvent marriage;
	RootsEvent marriageEnd;	/* due to divorce or anullment */
	String childrenString;	/* gets converted to children[] */
	RootsPerson children[];
	String adoptedChildrenString;
	RootsPerson adoptedChildren[];
	String addressString;
	RootsAddress address;

	public RootsUnion(RootsBase base) {
		super(base);
		marriage = new RootsEvent();
		marriageEnd = new RootsEvent();
	}

	public static RootsUnion ParseNcvLines(RootsBase base, String lines[]){
		RootsUnion r = new RootsUnion(base);
		int n = 0;
		while (n<lines.length) {
			Ncv nv = new Ncv(lines,n);
			n += nv.numLines;
			r.addNcv(nv);
		}
		return r;
	}
	/* Returns true if we recognized the name and added the value */
	protected boolean addNcv(Ncv nv) {
		if (super.addNcv(nv))
			return true;
		else if (nv.name.equals("N"))
			familyName = nv.value;
		else if (nv.name.equals("H"))
			husbandString = nv.value;
		else if (nv.name.equals("W"))
			wifeString = nv.value;
		else if (nv.name.equals("M"))
			marriage.date = nv.value;
		else if (nv.name.equals("MP"))
			marriage.place = nv.value;
		else if (nv.name.equals("MD"))
			marriageEnd.date = nv.value;
		else if (nv.name.equals("MDP"))
			marriageEnd.place = nv.value;
		else if (nv.name.equals("C"))
			childrenString = nv.value;
		else if (nv.name.equals("C.A"))
			adoptedChildrenString = nv.value;
		else if (nv.name.equals("ADDR"))
			addressString = nv.value;
		else
			return false;	/* not recognized */
		return true;		/* OK */
	}

	/* Fix up the links in a record that point to another record */
	protected void link() {
		RootsRecord r;
		RootsPerson pa[];
		super.link();
		if ((r=base.findRecord(husbandString))!=null)
			husband = (RootsPerson)r;
		if ((r=base.findRecord(wifeString))!=null)
			wife = (RootsPerson)r;
		if ((r=base.findRecord(addressString))!=null)
			address = (RootsAddress)r;
		if ((pa=RootsPerson.FindRecordList(base,childrenString))!=null)
			children = pa;
		if ((pa=RootsPerson.FindRecordList(
					base,adoptedChildrenString))!=null)
			adoptedChildren = pa;
	}

	/* Fix up the dates */
	protected void parseDates() {
		parseDate(marriage,"marriage");
		parseDate(marriageEnd,"marriageEnd");
	}

	/* Find a list of union */
	public static RootsUnion[] FindRecordList(RootsBase base, String s) {
		RootsRecord ra[];
		ra = base.findRecordList(s);
		if (ra==null)
			return null;
		RootsUnion a[] = new RootsUnion[ra.length];
		for (int i=0; i<ra.length; i++) {
			a[i] = (RootsUnion)(ra[i]);
		}
		return a;
	}

	/** Format the key line for a Gedcom record. */
	protected void formatGedcomKey(StringBuffer sb, int level) {
		formatGedcomKey(sb,level,"FAM");
	}
	/** Format the body of a Gedcom record */
	protected void formatGedcomBody(StringBuffer sb, int level) {
		formatGedcomEvent(sb,level,"MARR",marriage);
		formatGedcomReference(sb,level,"HUSB",husband);
		formatGedcomReference(sb,level,"WIFE",wife);
		RootsRecord c[] = findReference("children");
		if (c!=null) {
			for (int i=0; i<c.length; i++)
				formatGedcomReference(sb,level,"CHIL",c[i]);
		}
		formatGedcomReference(sb,level,"ADDR",address);
		super.formatGedcomBody(sb,level);
	}

	public String formatNcvType() {
		return("T:F");
	}
	public void formatNcvBody(StringBuffer sb) {
		formatNcvItem(sb,"N",familyName);
		formatNcvItem(sb,"H",husbandString);
		formatNcvItem(sb,"W",wifeString);
		formatNcvItem(sb,"M",marriage.date);
		formatNcvItem(sb,"MP",marriage.place);
		formatNcvItem(sb,"MD",marriageEnd.date);
		formatNcvItem(sb,"MDP",marriageEnd.place);
		formatNcvItem(sb,"C",childrenString);
		formatNcvItem(sb,"C.A",adoptedChildrenString);
		formatNcvItem(sb,"ADDR",addressString);
		super.formatNcvBody(sb);
	}
	public String formatOneLine(PrintOptions po) {
		StringBuffer sb = new StringBuffer();
		if (po.includeKeys)
			formatKey(sb);
		formatLabeledItems(sb,"m:",marriage.date,marriage.place," ");
		formatLabeledItems(sb,"x:",
			marriageEnd.date,marriageEnd.place," ");
		return sb.toString();
	}
	public String formatBrief(PrintOptions po) {
		StringBuffer sb = new StringBuffer();
		if (po.includeKeys)
			formatKey(sb);
		if (familyName==null)
			sb.append("(No family name)\n");
		else
			sb.append(familyName).append("\n");
		sb.append(" ");
		if (husband!=null) {
			if (po.includeKeys)
				husband.formatKey(sb);
			husband.formatName(sb);
		}
		else
			sb.append("?");
		sb.append("\n ");
		if (wife!=null) {
			if (po.includeKeys)
				wife.formatKey(sb);
			wife.formatName(sb);
		}
		else
			sb.append("?");
		sb.append("\n");
		formatBriefBody(sb);
		return sb.toString();
	}
	public String formatBriefBody() {
		StringBuffer sb = new StringBuffer();
		formatBriefBody(sb);
		return sb.toString();
	}
	public void formatBriefBody(StringBuffer sb) {
		formatLabeledItems(sb,"m: ",marriage.date,marriage.place,"\n");
		formatLabeledItems(sb,"e: ",
			marriageEnd.date,marriageEnd.place,"\n");
		formatNote1(sb,"","\n");
	}

	/** Fill in the layout information for a ctree,
	 * including all other boxes for the ctree layout.
	 */
	public void layoutCtree(Layout layout, PrintOptions po) {
		/* Do our own box. */
		layoutBox(layout,po,key);

		/* Add these other boxes by reference. */
		layoutBoxReference(layout,po,"husband");
		layoutBoxReference(layout,po,"wife");
		layoutBoxReference(layout,po,"children");
		layoutBoxReference(layout,po,"adoptedChildren");
	}

	/** Fill in the layout information for a ctree.
	 * @param layout The layout to which we add our box or boxes.
	 * @param po Options.
	 * @param markKey Put a marker by the record with this key.  This can
	 *        be either this union or a child.
	 */
	protected void layoutBox(Layout layout, PrintOptions po,
						String markKey){
		RootsBox box = new RootsBox(layout,key);
		String familyText = formatBrief(po) + "=====\n";
			/* TBD - make separator line the right length */
		if (key.equals(markKey))
			familyText = "*"+familyText;
		box.addSection(familyText,null);
		if (children!=null && children.length>0) {
			for (int i=0; i<children.length; i++) {
				RootsRecord r = children[i];
				String childText = r.formatBrief(po);
				if (r.key.equals(markKey))
					childText = "*" + childText;
				box.addSection(childText,r.key);
			}
		}
		if (adoptedChildren!=null && adoptedChildren.length>0) {
			for (int i=0; i<adoptedChildren.length; i++) {
				RootsRecord r = adoptedChildren[i];
				String childText = r.formatBrief(po);
				if (r.key.equals(markKey))
					childText = "*" + childText;
				box.addSection(childText,r.key);
			}
		}
		box.setParentKeys(husbandString,wifeString);
		box.done();
	}

	public RootsRecord[] findReference(String what) {
		if (what==null)
			return null;
		if (what.equals("father")||what.equals("husband")) {
			if (husband==null)
				return null;
			return husband.asArray();
		}
		if (what.equals("mother")||what.equals("wife")) {
			if (wife==null)
				return null;
			return wife.asArray();
		}
		if (what.equals("address")) {
			if (address==null)
				return null;
			return address.asArray();
		}
		if (what.startsWith("children"))
			return findListReference(children,what.substring(8));
		if (what.startsWith("child"))
			return findListReference(children,what.substring(5));
		if (what.startsWith("adoptedChildren"))
			return findListReference(
				adoptedChildren,what.substring(15));
		if (what.startsWith("adoptedChild"))
			return findListReference(
				adoptedChildren,what.substring(12));
		return super.findReference(what);
	}

	/** Return true if the specified field matches the given pattern.
	 * @param fieldName the name of the field to match
	 * <ul>
	 * <li>"place" to match the name of any locations
	 *     (marriage, marriageEnd) against the pattern.
	 * </ul>
	 * @param pattern the pattern to match against.
	 */
	public boolean fieldMatches(String fieldName, String pattern) {
		if (fieldName.equals("place")) {
			if (PatternMatch.stringMatches(marriage.place,pattern))
				return true;
			if (PatternMatch.stringMatches(
						marriageEnd.place,pattern))
				return true;
			return super.fieldMatches(fieldName,pattern);
				/* super may have additional places to match */
		}
		return super.fieldMatches(fieldName,pattern);
	}

	/** Print a line to the default output. */
	public void println(String s) {
		base.println(s);
	}
}

/* end */
