/* TextOrLayoutPanel.java
 *
 * Jim McBeath, July 14, 1998
 */

package jimmc.roots;

import jimmc.treepar.Layout;

import java.awt.CardLayout;
import javax.swing.JPanel;

/** Displays either a text string or a treepar layout.
 */
public class TextOrLayoutPanel extends JPanel {
	/** The awt layout for the panel. */
	protected CardLayout cardLayout;

	/** Our text field. */
	protected ScrolledText textField;

	/** Our layout field. */
	protected ScrolledLayout layoutArea;

	/** Create a new panel. */
	public TextOrLayoutPanel() {
		super();
		setLayout();
		addTextPanel();
		addLayoutPanel();
	}

	/** Set up our awt layout manager. */
	protected void setLayout() {
		cardLayout = new CardLayout();
		setLayout(cardLayout);
	}

	/** Add a scrolling text panel. */
	protected void addTextPanel() {
		textField = new ScrolledTextWithPopup(500,300) {
			public void processSelect(String key) {
				processTextSelect(key);
			}
		};
		add(textField,"text");
		cardLayout.show(this,"text");
	}

	/** Add a treepar layout field. */
	protected void addLayoutPanel() {
		layoutArea = new ScrolledLayoutWithPopup(500,300) {
			public void processSelect(String key) {
				processTextSelect(key);
			}
		};
		add(layoutArea,"layout");
	}

	/** Process a selection in the text field. */
	protected void processTextSelect(String key) {
		/* let subclass override */
	}

	/** Set the text into the text field and show it. */
	public void setText(String text) {
		if (text==null)
			text = "";
		textField.setText(text);
		cardLayout.show(this,"text");
//System.out.println("show text");
		doLayout();
	}

	/** Get the text from the text field. */
	public String getText() {
		return textField.getText();
	}

	/** Set the treepar layout and show it. */
	public void setLayout(Layout layout) {
		layoutArea.setLayout(layout);
		cardLayout.show(this,"layout");
//System.out.println("show layout");
		doLayout();
	}
}

/* end */
