/* LayoutPanel.java
 *
 * Jim McBeath, July12, 1998
 */

package jimmc.treepar;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import javax.swing.JPanel;

/** LayoutPanel is a panel designed to contain and display a Layout.
 */
public class LayoutPanel extends JPanel {
	/** Our layout. */
	protected Layout layout;

	/** The most recently used LayoutGraphics. */
	protected LayoutGraphics layoutGraphics;

	/** Create a LayoutPanel for a Layout. */
	public LayoutPanel(Layout layout) {
		super();
		setLayout(layout);
	}

	/** Set the treepar layout for this panel. */
	public void setLayout(Layout layout) {
		this.layout = layout;
	}

	/** Make the preferred size big enough for a 1-1 scale
	 * with a margin. */
	public Dimension getPreferredSize() {
		if (layout==null)
			return new Dimension(10,10);
		Rectangle lb = layout.getBounds();
		/* Add a two-line margin around the layout */
		int margin = 2*layout.lineHeight;
		lb.width += 2*margin;
		lb.height += 2*margin;
		return new Dimension(lb.width,lb.height);
	}

	/** Paint our window. */
	public void paint(Graphics g) {
		LayoutGraphics lg = new LayoutGraphics(g);
		layoutGraphics = lg;
//System.out.println("painting layout");

		Rectangle lb = layout.getBounds();
		/* Add a two-line margin around the layout */
		int margin = 2*layout.lineHeight;
		lb.x -= margin;
		lb.y -= margin;
		lb.width += 2*margin;
		lb.height += 2*margin;
		lg.setLayoutBounds(lb);

		Dimension d = getSize();
//g.drawLine(0,0,d.width-1,d.height-1);
		Rectangle r = new Rectangle(0,0,d.width,d.height);
		lg.setWindowBounds(r);

		/* Set up the right font size for our characters. */
		lg.setFont(layout.lineHeight);

		boolean drawBox = false;
		if (System.getProperty("TREEPAR_DRAW_BOX")!=null)
			drawBox = true;
		if (drawBox) {
			/* Draw an outline box about 4% larger than the 
			 * actual layout */
			int boxMargin;
			Rectangle layoutBounds = layout.getBounds();
			if (layoutBounds.width>layoutBounds.height)
				boxMargin = layoutBounds.height/24;
			else
				boxMargin = layoutBounds.width/24;
			Point p = new Point(layoutBounds.x - boxMargin,
					    layoutBounds.y - boxMargin);
			d.width = layoutBounds.width + 2*boxMargin;
			d.height = layoutBounds.height + 2*boxMargin;
			lg.drawBox(p,d);
		}

		/* Draw the layout. */
		layout.paint(lg);
	}

	/** Get the line of text at the specified screen point. */
	public LayoutHitText getHitText(int screenX, int screenY) {
		if (layoutGraphics==null || layout==null)
			return null;
		int x = layoutGraphics.reverseTransformX(screenX);
		int y = layoutGraphics.reverseTransformY(screenY);
		return layout.getHitText(x,y);
	}
}

/* end */
