/* Test.java
 *
 * Jim McBeath, July 11, 1998
 */

package jimmc.treepar;

import jimmc.roots.ButtonAction;
import jimmc.roots.GridBagger;

import java.awt.Dimension;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.Graphics;
import java.awt.Panel;
import java.io.PrintWriter;

/** This class is used for testing and debugging the treepar package. */
public class Test {
	/** Our layout. */
	protected Layout layout;

	/** Allocate a test object. */
	public Test() {
		/* nothing here. */
	}

	/** It all starts with main. */
	public static void main(String[] args) {
		new Test().doMain(args);
	}

	/** Do all the work for main. */
	protected void doMain(String[] args) {
		System.out.println("Begin");
		layout = new Layout(true); /* layout with vertical rows*/
		fillLayout();		/* put in all the boxes */

		int lastStep = Layout.STEP_BOUNDS;
		/* TBD - for debugging, can do less than the whole thing. */
		layout.placeAndRoute(lastStep);		/* do it all */

		dumpLayout();

		showPlot();		/* put up a graphics window */
	}

	/** Dump the layout to stdout. */
	protected void dumpLayout() {
		PrintWriter f = new PrintWriter(System.out);
		layout.dump(f);
		f.flush();
	}

	/** Fill in the layout with boxes. */
	protected void fillLayout() {
		Box box;
		Connector conn;

		box = new Box(layout,"b1");
		box.setSize(40,60);
		box.setText("This is b1");
		conn = new Connector(box,"net1");
		conn.setSide(Layout.EAST);
		conn.setPosition(40,30);

		box = new Box(layout,"b2");
		box.setSize(45,50);
		box.setText("Box b2");
		conn = new Connector(box,"net1");
		conn.setSide(Layout.WEST);
		conn.setPosition(0,20);
	}

	/** Put up a window in which the layout is drawn. */
	protected void showPlot() {
		System.out.println("ready to plot");
		Frame f = new Frame("Test");
		WindowAdapter wd = new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		};
		f.addWindowListener(wd);
		GridBagger gb = new GridBagger(f);

		gb.add(new ButtonAction("Exit") {
			public void action() {
				System.exit(0);
			}
		});

		gb.nextRow();
		gb.gbc.weightx = 1;
		gb.gbc.weighty = 1;
		LayoutPanel p = new LayoutPanel(layout);
		gb.gbc.fill = GridBagConstraints.BOTH;
		gb.add(p);

		f.pack();
		f.show();
	}
}

/* end */
